@@ -1,4 +1,5 @@
-{-# LANGUAGE FlexibleContexts, NoImplicitPrelude, DeriveTraversable, NamedFieldPuns, DerivingVia, DeriveGeneric, OverloadedRecordDot #-}
+{-# LANGUAGE FlexibleContexts, NoImplicitPrelude, DeriveTraversable, NamedFieldPuns #-}
+{-# LANGUAGE DerivingVia, DeriveGeneric, OverloadedRecordDot, LambdaCase #-}
 
 module Conflict
     ( Conflict(..), Sides(..), SrcContent(..)
@@ -7,11 +8,11 @@ module Conflict
     , parse
     ) where
 
-import Control.Monad.State (MonadState, state, evalStateT)
+import Control.Monad.State (MonadState, evalStateT, state)
 import Control.Monad.Writer (runWriter, tell)
 import Data.Maybe (fromMaybe)
-import Generic.Data (Generically1(..))
 import GHC.Generics (Generic1)
+import Generic.Data (Generically1(..))
 
 import Prelude.Compat
 
@@ -19,19 +20,22 @@ data Sides a = Sides
     { sideA :: a
     , sideBase :: a
     , sideB :: a
-    } deriving (Functor, Foldable, Traversable, Show, Eq, Ord, Generic1)
-    deriving Applicative via Generically1 Sides
+    }
+    deriving (Functor, Foldable, Traversable, Show, Eq, Ord, Generic1)
+    deriving (Applicative) via Generically1 Sides
 
 data SrcContent = SrcContent
     { lineNo :: Int
     , content :: String
-    } deriving Show
+    }
+    deriving (Show)
 
 data Conflict = Conflict
-    { markers   :: Sides SrcContent -- The markers at the beginning of sections
-    , markerEnd :: SrcContent       -- The ">>>>>>>...." marker at the end of the conflict
-    , bodies    :: Sides [String]
-    } deriving Show
+    { markers :: Sides SrcContent -- The markers at the beginning of sections
+    , markerEnd :: SrcContent -- The ">>>>>>>...." marker at the end of the conflict
+    , bodies :: Sides [String]
+    }
+    deriving (Show)
 
 setBodies :: (Sides [String] -> Sides [String]) -> Conflict -> Conflict
 setBodies f c = c{bodies = f c.bodies}
@@ -44,7 +48,7 @@ setStrings = setEachBody . map
 
 prettyLines :: Conflict -> [String]
 prettyLines c =
-    concat ((:) <$> ((.content) <$> c.markers) <*> c.bodies) <> [c.markerEnd.content]
+    concat ((:) . content <$> c.markers <*> c.bodies) <> [c.markerEnd.content]
 
 pretty :: Conflict -> String
 pretty = unlines . prettyLines
@@ -81,20 +85,21 @@ tryReadUpToMarker c mCount =
 readUpToMarker ::
     MonadState [SrcContent] m =>
     Char -> Maybe Int -> m ([SrcContent], SrcContent)
-readUpToMarker c mCount = do
-    res <- tryReadUpToMarker c mCount
-    case res of
-        (ls, Just h)  -> pure (ls, h)
-        (ls, Nothing) ->
-            error $ concat
-            [ "Parse error: failed reading up to marker: "
-            , show c, ", got:"
-            , concatMap (\l -> "\n" ++ show l.lineNo ++ "\t" ++ l.content) $ take 5 ls
-            ]
+readUpToMarker c mCount =
+    tryReadUpToMarker c mCount >>=
+    \case
+    (ls, Just h) -> pure (ls, h)
+    (ls, Nothing) ->
+        error $ concat
+        [ "Parse error: failed reading up to marker: "
+        , show c, ", got:"
+        , concatMap (\l -> "\n" ++ show l.lineNo ++ "\t" ++ l.content) $ take 5 ls
+        ]
 
 parseConflict :: MonadState [SrcContent] m => SrcContent -> m Conflict
 parseConflict markerA =
-    do  (linesA   , markerBase) <- readUpToMarker '|' markerCount
+    do
+        (linesA, markerBase) <- readUpToMarker '|' markerCount
         (linesBase, markerB)    <- readUpToMarker '=' markerCount
         (linesB   , markerEnd)  <- readUpToMarker '>' markerCount
         pure Conflict
@@ -110,13 +115,15 @@ parseFromNumberedLines =
     snd . runWriter . evalStateT loop
     where
         loop =
-            do  (ls, mMarkerA) <- tryReadUpToMarker '<' Nothing
+            do
+                (ls, mMarkerA) <- tryReadUpToMarker '<' Nothing
                 tell $ map (Left . (.content)) ls
                 case mMarkerA of
                     Nothing -> pure ()
                     Just markerA ->
-                        do  tell . pure . Right =<< parseConflict markerA
+                        do
+                            tell . pure . Right =<< parseConflict markerA
                             loop
 
 parse :: String -> [Either String Conflict]
-parse = parseFromNumberedLines . zipWith SrcContent [1..] . lines
+parse = parseFromNumberedLines . zipWith SrcContent [1 ..] . lines
