@@ -3,20 +3,23 @@
 module Resolution
     ( Result(..)
     , NewContent(..)
-    , Untabify(..)
     , resolveContent
     , fullySuccessful
     ) where
 
 import           Conflict (Conflict(..), Sides(..))
 import qualified Conflict
+import           Control.Monad (guard)
 import           Data.Foldable (Foldable(..))
+import           Data.List (isPrefixOf)
+import           Data.List.Split (splitWhen)
 import           Generic.Data (Generic, Generically(..))
+import           ResolutionOpts
 
 import           Prelude.Compat
 
 data Resolution
-    = NoResolution
+    = NoResolution Conflict
     | Resolution String
     | PartialResolution String
 
@@ -27,18 +30,29 @@ resolveGen (Sides a base b)
     | a == b = Just a
     | otherwise = Nothing
 
-resolveConflict :: Conflict -> Resolution
-resolveConflict c =
-    case resolveGen c.bodies of
-    Just r -> Resolution $ unlines r
-    Nothing
-        | matchTop > 0 || matchBottom > 0 ->
-            PartialResolution $ unlines $
-            take matchTop c.bodies.sideA ++
-            Conflict.prettyLines (Conflict.setEachBody unmatched c) ++
-            takeEnd matchBottom c.bodies.sideA
-        | otherwise -> NoResolution
+resolveGenLines :: Eq a => ResolutionOptions -> Sides [a] -> Maybe [a]
+resolveGenLines opts sides@(Sides a base b) =
+    case guard opts.trivial >> resolveGen sides of
+    Just x -> Just x
+    _ | opts.addedLines ->
+        case addedBothSides a b <> addedBothSides b a of
+        [x] -> Just x
+        _ -> Nothing
+    _ -> Nothing
     where
+        n = length base
+        addedBothSides x y = [x <> drop n y | drop (length x - n) x == base && take n y == base]
+
+resolveConflict :: ResolutionOptions -> Conflict -> Resolution
+resolveConflict opts c
+    | matchTop == 0 && matchBottom == 0 || not opts.reduce =
+        maybe (NoResolution c) (Resolution . unlines) (resolveGenLines opts c.bodies)
+    | otherwise =
+        case resolveGenLines opts reduced.bodies of
+        Just x -> Resolution $ formatRes x
+        Nothing -> PartialResolution $ formatRes $ Conflict.prettyLines reduced
+    where
+        formatRes mid = unlines $ take matchTop c.bodies.sideA <> mid <> takeEnd matchBottom c.bodies.sideA
         match base a b
             | null base = lengthOfCommonPrefix a b
             | otherwise = minimum $ map (lengthOfCommonPrefix base) [a, b]
@@ -48,6 +62,7 @@ resolveConflict c =
         dropEnd count xs = take (length xs - count) xs
         takeEnd count xs = drop (length xs - count) xs
         unmatched xs = drop matchTop $ dropEnd matchBottom xs
+        reduced = Conflict.setEachBody unmatched c
 
 lengthOfCommonPrefix :: Eq a => [a] -> [a] -> Int
 lengthOfCommonPrefix x y = length $ takeWhile id $ zipWith (==) x y
@@ -73,8 +88,6 @@ data NewContent = NewContent
     deriving Generic
     deriving (Semigroup, Monoid) via Generically NewContent
 
-newtype Untabify = Untabify { mUntabifySize :: Maybe Int }
-
 untabifyStr :: Int -> String -> String
 untabifyStr size =
     go 0
@@ -86,8 +99,8 @@ untabifyStr size =
         go !col (x:rest) = x : go (cyclicInc col) rest
         go _ [] = []
 
-untabify :: Int -> Conflict -> Conflict
-untabify = Conflict.setStrings . untabifyStr
+untabifyConflict :: Int -> Conflict -> Conflict
+untabifyConflict = Conflict.setStrings . untabifyStr
 
 data LineEnding = LF | CRLF | Mixed
     deriving (Eq, Ord)
@@ -96,9 +109,9 @@ lineEnding :: String -> LineEnding
 lineEnding x@(_:_) | last x == '\r' = CRLF
 lineEnding _ = LF
 
-lineEndings :: [String] -> LineEnding
-lineEndings [] = Mixed
-lineEndings xs =
+inferLineEndings :: [String] -> LineEnding
+inferLineEndings [] = Mixed
+inferLineEndings xs =
     foldl1 f (map lineEnding xs)
     where
         f Mixed _ = Mixed
@@ -120,21 +133,45 @@ lineBreakFix c
         Just CRLF -> Conflict.setStrings makeCr c
         _ -> c
     where
-        endings = fmap lineEndings c.bodies
+        endings = fmap inferLineEndings c.bodies
         removeCr x@(_:_) | last x == '\r' = init x
         removeCr x = x
         makeCr x@(_:_) | last x == '\r' = x
         makeCr x = x <> "\r"
 
-resolveContent :: Untabify -> [Either String Conflict] -> NewContent
-resolveContent (Untabify mTabSize) =
+formatResolution :: Resolution -> NewContent
+formatResolution (NoResolution c) = NewContent (Result 0 0 1) (Conflict.pretty c)
+formatResolution (Resolution trivialLines) = NewContent (Result 1 0 0) trivialLines
+formatResolution (PartialResolution newLines) = NewContent (Result 0 1 0) newLines
+
+resolveContent :: ResolutionOptions -> [Either String Conflict] -> NewContent
+resolveContent opts =
     foldMap go
     where
-        untabified = maybe id untabify mTabSize
         go (Left line) = NewContent mempty (unlines [line])
-        go (Right conflict) =
-            case (resolveConflict . lineBreakFix . untabified) conflict of
-            NoResolution               -> NewContent (Result 0 0 1)
-                                          (Conflict.pretty (untabified conflict))
-            Resolution trivialLines    -> NewContent (Result 1 0 0) trivialLines
-            PartialResolution newLines -> NewContent (Result 0 1 0) newLines
+        go (Right conflict)
+            | opts.splitMarkers =
+                r <> splitProgress
+            | otherwise = resolve conflict
+                where
+                    s = splitConflict conflict
+                    r = foldMap resolve s
+                    splitProgress =
+                        case s of
+                        (_:_:_) -> NewContent (Result 0 1 0) mempty
+                        _ -> mempty
+        resolve conflict =
+            formatResolution $ resolveConflict opts $
+            (if opts.lineEndings then lineBreakFix else id) $
+            maybe id untabifyConflict opts.untabify conflict
+
+splitConflict :: Conflict -> [Conflict]
+splitConflict c =
+    maybe [c] ((\s -> c{bodies=s}) <$>) $
+    matchSplits $ splitWhen (isPrefixOf "~~~~~~~") <$> c.bodies
+
+matchSplits :: Sides [a] -> Maybe [Sides a]
+matchSplits (Sides (a:as) (b:bs) (c:cs)) =
+    (Sides a b c :) <$> matchSplits (Sides as bs cs)
+matchSplits (Sides [] [] []) = Just []
+matchSplits _ = Nothing
