@@ -1,10 +1,11 @@
 {-# LANGUAGE NoImplicitPrelude #-}
 
 module StrUtils
-    ( stripNewline, ensureNewline, unprefix
+    ( (</>), stripNewline, ensureNewline
     ) where
 
-import           Data.List (isPrefixOf, isSuffixOf)
+import           Data.List (isSuffixOf)
+import qualified System.FilePath as FilePath
 
 import           Prelude.Compat
 
@@ -21,7 +22,6 @@ ensureNewline str = str ++ suffix
             | "\n" `isSuffixOf` str = ""
             | otherwise = "\n"
 
-unprefix :: Eq a => [a] -> [a] -> Maybe [a]
-unprefix prefix str
-    | prefix `isPrefixOf` str = Just (drop (length prefix) str)
-    | otherwise = Nothing
+(</>) :: FilePath -> FilePath -> FilePath
+"." </> p = p
+d </> p = d FilePath.</> p
